
var $ = require('ore').query;

exports.createPowerOfTwoImage = function(image) {
    // Derived from http://www.khronos.org/webgl/wiki/WebGL_and_OpenGL_Differences
    if (!isPowerOfTwo(image.width) || !isPowerOfTwo(image.height)) {
        var canvas = document.createElement("canvas");
        canvas.width = nextHighestPowerOfTwo(image.width);
        canvas.height = nextHighestPowerOfTwo(image.height);
        var ctx = canvas.getContext("2d");
        // Draw initially at to bottom left
        var y = canvas.height-image.height;
        ctx.drawImage(image, 0, 0, image.width, image.height, 0, y, image.width, image.height);
        
        // Draw mirrored on right
        if (image.width < canvas.width) {
            ctx.save();
            ctx.translate(image.width*2, 0);
            ctx.scale(-1,1);
            ctx.drawImage(image, 0, 0, image.width, image.height, 0, y, image.width, image.height);
            ctx.restore();
        }
        // Draw mirrored above
        if (image.height < canvas.height) {
            ctx.scale(1,-1);
            ctx.translate(0,-y*2);
            ctx.drawImage(image, 0, 0, image.width, image.height, 0, y, image.width, image.height);
        }
        return canvas;
    } else {
        return image;
    }
};
  
var nextHighestPowerOfTwo = exports.nextHighestPowerOfTwo = function(x) {
    --x;
    for (var i = 1; i < 32; i <<= 1) {
        x = x | x >> i;
    }
    return x + 1;
}

function isPowerOfTwo(x) {
    return (x & (x - 1)) == 0;
}

// *************************************************************************************************

exports.uniformMap = {
    /* gl.FLOAT */ 5126: function(gl,u,v) { gl.uniform1f(u,v);},
    /* gl.FLOAT_VEC2 */ 35664: function(gl,u,v) { gl.uniform2f(u,v[0],v[1]);},
    /* gl.FLOAT_VEC3 */ 35665: function(gl,u,v) { gl.uniform3f(u,v[0],v[1],v[2]);},
    /* gl.FLOAT_VEC4 */ 35666: function(gl,u,v) { gl.uniform4f(u,v[0],v[1],v[2],v[3]);},
    /* gl.INT */ 5124: function(gl,u,v) { gl.uniform1i(u,v);},
    /* gl.INT_VEC2 */ 35667: function(gl,u,v) { gl.uniform2i(u,v[0],v[1]);},
    /* gl.INT_VEC3 */ 35668: function(gl,u,v) { gl.uniform3i(u,v[0],v[1],v[2]);},
    /* gl.INT_VEC4 */ 35669: function(gl,u,v) { gl.uniform4i(u,v[0],v[1],v[2],v[3]);},
    /* gl.BOOL */ 35670: function(gl,u,v) { gl.uniform1i(u,v);},
    /* gl.BOOL_VEC2 */ 35671: function(gl,u,v) { gl.uniform2i(u,v[0],v[1]);},
    /* gl.BOOL_VEC3 */ 35672: function(gl,u,v) { gl.uniform3i(u,v[0],v[1],v[2]);},
    /* gl.BOOL_VEC4 */ 35673: function(gl,u,v) { gl.uniform4i(u,v[0],v[1],v[2],v[3]);},
    /* gl.MAT2 */ 35674: function(gl,u,v) { gl.uniformMatrix2fv(u, false, new Float32Array(v));},
    /* gl.MAT3 */ 35675: function(gl,u,v) { gl.uniformMatrix3fv(u, false, new Float32Array(v));},
    /* gl.MAT4 */ 35676: function(gl,u,v) { gl.uniformMatrix4fv(u, false, new Float32Array(v));},
    /* gl.SAMPLER2D */ 35678: function(gl,u,v) { gl.uniform1i(u, false, new Float32Array(v));},
};

// *************************************************************************************************

var shader = exports.shader = function(fn) {
    return function(id, uniforms) {
        if (typeof(id) == 'object') {
            uniforms = id;
            id = null;
        }
        return {
            id: id,
            fn: fn,
            uniforms: uniforms,
            set: function(name, value) {
                this.uniforms[name] = value;
            }
        };
    };
}

exports.defaultShaderHeader = 'precision highp float;';
